# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: result.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Object that stores the result of a search for one or more packages
and can return specific options from them, such as cflags, libs, etc.
Will search recursively to find all required packages.

"""

__version__ = "$Revision: $"
# $Source$

import sys

from pykg_config.pkgsearcher import PkgSearcher
from pykg_config.errorprinter import ErrorPrinter
from pykg_config.exceptions import PykgConfigError
from pykg_config.packagespeclist import parse_package_spec_list
from pykg_config.options import Options

##############################################################################
# Exceptions

class PackageConflictError(PykgConfigError):
    """A package listed in a conflict list was found.

    Attributes:
        pkgname -- The name of the package with the conflict.
        conflict -- The package causing the conflict.

    """
    def __init__(self, pkgname, conflict):
        self.pkgname = pkgname
        self.conflict = conflict

    def __str__(self):
        return '{0} conflicts with {1}'.format(self.pkgname, self.conflict)


class NoPackagesSpecifiedError(PykgConfigError):
    """No packages were specified in a list of packages to search for."""
    pass

##############################################################################
# Private utility functions

def _filter_duplicates(source):
    # This maintains the order.
    ret = []
    seen = { }
    for it in reversed(source):
         if it not in seen:
              ret.append(it)
              seen[it] = 1
    ret.reverse()
    return ret

##############################################################################
# PkgCfgResult object

class PkgCfgResult:
    def __init__(self, globals):
        # Different platforms may use different flags and extensions
        if sys.platform == 'win32' and Options().get_option('use_msvc_syntax'):
            self.lib_path_flag = '/libpath:'
            self.lib_flag = ''
        else:
            self.lib_path_flag = '-L'
            self.lib_flag = '-l'
        # Can't use a dictionary for the loaded packages list because the
        # ordering must be maintained (when Python 3.1 is standard, switch to
        # collections.OrderedDict).
        self.packages = []
        self.searched_packages = []
        self.searcher = PkgSearcher()
        self.globals = globals

    def __str__(self):
        packages = ['%s-%s' % (pkg[0], pkg[1].properties['version']) \
                    for pkg in self.packages]
        return str(packages)

    def find_packages(self, pkglist, recurse):
        """Find all packages specified in pkglist, which must be a
        textual list of package specifications in the form
        'package operator version' (where operator is a comparison
        operator such as <= or ==).

        """
        self.searched_packages = parse_package_spec_list(pkglist)
        if not self.searched_packages:
            raise NoPackagesSpecifiedError
        self._load_dependencies(self.searched_packages, recurse)

    def dump_package(self):
        for pkg in self.searched_packages:
            print(self._get_loaded_package(pkg.name))

    def get_big_i_flags(self):
        result = []
        for name, pkg in self.packages:
            result += ['-I' + dir for dir in pkg.properties['include_dirs']]
        return self._format_list(_filter_duplicates(result))

    def get_other_i_flags(self):
        result = []
        for name, pkg in self.packages:
            result += pkg.properties['other_cflags']
        return self._format_list(_filter_duplicates(result))

    def get_cflags(self):
        result = []
        for name, pkg in self.packages:
            result += pkg.properties['other_cflags']
            result += ['-I' + dir for dir in pkg.properties['include_dirs']]
        return self._format_list(_filter_duplicates(result))

    def get_l_flags(self):
        result = []
        for name, pkg in self.packages:
            result += [self.lib_flag + lib for lib in pkg.properties['libs']]
            if Options().get_option('private_libs'):
                result += [self.lib_flag + lib \
                                for lib in pkg.properties['private.libs']]
        return self._format_list(_filter_duplicates(result))

    def get_big_l_flags(self):
        result = []
        for name, pkg in self.packages:
            result += [self.lib_path_flag + path \
                        for path in pkg.properties['libpaths']]
            if Options().get_option('private_libs'):
                result += [self.lib_path_flag + path \
                                for path in pkg.properties['private.libpaths']]
        return self._format_list(_filter_duplicates(result))

    def get_other_l_flags(self):
        result = []
        for name, pkg in self.packages:
            result += pkg.properties['otherlibs']
            if Options().get_option('private_libs'):
                result += pkg.properties['private.otherlibs']
        return self._format_list(_filter_duplicates(result))

    def get_all_lib_flags(self):
        result = []
        for name, pkg in self.packages:
            result += pkg.properties['otherlibs']
            if Options().get_option('private_libs'):
                result += pkg.properties['private.otherlibs']
            result += [self.lib_path_flag + path \
                        for path in pkg.properties['libpaths']]
            if Options().get_option('private_libs'):
                result += [self.lib_path_flag + path \
                                for path in pkg.properties['private.libpaths']]
            result += [self.lib_flag + lib for lib in pkg.properties['libs']]
            if Options().get_option('private_libs'):
                result += [self.lib_flag + lib \
                                for lib in pkg.properties['private.libs']]
        return self._format_list(_filter_duplicates(result))

    def get_variable_value(self, variable):
        """Find a variable definition in the list of packages, going through
        the list of packages in the command line order first before trying
        dependencies."""
        for pkg in self.searched_packages:
            loaded_pkg = self._get_loaded_package(pkg.name)
            if loaded_pkg.variables.has_key(variable):
                return loaded_pkg.variables[variable]
        # Not in the searched-for packages, so try the dependencies
        for name, pkg in self.packages:
            if pkg.variables.has_key(variable):
                return pkg.variables[variable]
        return None

    def known_packages_list(self):
        """Get a list of all the packages found on the system."""
        return self.searcher.known_packages_list()

    def have_uninstalled(self):
        for name, pkg in self.packages:
            if pkg.uninstalled:
                return True
        return False

    def get_package_version(self, name):
        pkg = self._get_loaded_package(name)
        if pkg:
            return pkg.properties['version']
        return None

    def get_searched_pkgs_versions(self):
        result = []
        for pkg in self.searched_packages:
            loaded_pkg = self._get_loaded_package(pkg.name)
            result.append(loaded_pkg.properties['version'])
        return result

    def get_searched_pkg_list(self):
        return self.searched_packages

    def _format_list(self, result_list):
        result = ''
        for item in ['{0} '.format(x) for x in result_list]:
            result += item
        return result[:-1] # Strip the last space

    def _load_dependencies(self, dependencies, recurse):
        # Search for and load all packages listed in dependencies,
        # attempting to recursively load all their required packages
        # and checking for conflicts as well.
        # For d in deps:
        for dep in dependencies:
        # 0. Check if this package is already loaded
            if self._get_loaded_package(dep.name):
                continue
        # 1. Search for a package matching the spec
            pkg = self.searcher.search_for_package(dep, self.globals)
        # 2. Check for conflicts
            for conflict in pkg.properties['conflicts']:
                try:
                    ErrorPrinter().debug_print('Searching for conflict %s',
                                               conflict.name)
                    self.searcher.search_for_package(conflict)
                except PackageNotFoundError:
                    # If the conflict was not found, move on to the next
                    ErrorPrinter().debug_print('Conflict not found.')
                    continue
                # Conflict was found - what to do?
                raise PackageConflictError(pkg.name, conflict)
        # 3. Sanity check this package
            pkg.sanity_check()
        # 4. Add this package to the dictionary now to avoid infinite recursion
            self._add_package(dep.name, pkg)
        # 5. Recursively load requireds
            if recurse:
                # requires.private includes requires
                self._load_dependencies(pkg.properties['requires.private'], recurse)

    def _add_package(self, name, newpkg):
        ErrorPrinter().debug_print('Adding %s to list of packages as %s',
                                   (newpkg.filename, name))
        self.packages.append((name, newpkg))

    def _get_loaded_package(self, name):
        for pkg in self.packages:
            if pkg[0] == name:
                return pkg[1]
        return None


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: options.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Singleton containing option values."""

__version__ = "$Revision: $"
# $Source$

import sys

from pykg_config.exceptions import PykgConfigError

##############################################################################
# Exceptions

class NoSuchOptionError(PykgConfigError):
    """The requested option has not been set.

    Attributes:
        option -- The option that doesn't exist."""
    def __init__(self, option):
        self.option = option

    def __str__(self):
        return self.option


##############################################################################
# Options singleton class

class Options(object):
    def __new__(cls, *p, **k):
        if not '_the_instance' in cls.__dict__:
            cls._the_instance = object.__new__(cls)
        return cls._the_instance

    def init_options(self):
        self.options = {'use_msvc_syntax': True,
                        'dont_define_prefix': False,
                        'prefix_variable': 'prefix',
                        'verbose': False,
                        'pc_path': '',
                        'uninstalled_only': False,
                        'prefer_uninstalled': True,
                        'pc_sysrootdir': '',
                        'pc_topbuilddir': '',
                        'print_errors': True,
                        'short_errors': False,
                        'error_dest': sys.stderr,
                        'debug': False,
                        'search_string': '',
                        'private_libs': False,
                        'forbidden_libdirs': [],
                        'forbidden_cflags': [],
                        'is_64bit': False,
                        'full_compatibility': False,
                        'normalise_paths': True}

    def set_option(self, option, value):
        if not hasattr(self, 'options'):
            self.init_options()
        self.options[option] = value

    def get_option(self, option):
        if not hasattr(self, 'options'):
            self.init_options()
        if not option in self.options:
            raise NoSuchOptionError(option)
        return self.options[option]


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: dependency.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Dependency object.

This object is used to store other packages that are depended on by a
package, both positive (need) and negative (conflict).

"""

__version__ = "$Revision: $"
# $Source$

from pykg_config.operators import *

##############################################################################
# Dependency class

class Dependency:
    def __init__(self, name, operator, version):
        self.name = name
        self.operator = operator
        self.version = version

    def __eq__(self, other):
        if self.name == other.name and \
           self.operator == other.operator and \
           self.version == other.version:
               return True
        return False

    def __ne__(self, other):
        if self.name != other.name or \
           self.operator != other.operator or \
           self.version != other.version:
               return True
        return False

    def __str__(self):
        if self.version.is_empty():
            return self.name
        return self.name + operator_to_text(self.operator) + str(self.version)

    def meets_requirement(self, other_version):
        if self.operator == ALWAYS_MATCH:
            return True
        elif self.operator == LESS_THAN:
            return other_version.__lt__(self.version)
        elif self.operator == LESS_THAN_EQUAL:
            return other_version.__le__(self.version)
        elif self.operator == EQUAL:
            return other_version.__eq__(self.version)
        elif self.operator == NOT_EQUAL:
            return other_version.__ne__(self.version)
        elif self.operator == GREATER_THAN_EQUAL:
            return other_version.__ge__(self.version)
        elif self.operator == GREATHER_THAN:
            return other_version.__gt__(self.version)


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: operators.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Comparison operators."""

__version__ = "$Revision: $"
# $Source$

from pykg_config.exceptions import ParseError

##############################################################################
# Exceptions

class BadOperatorError(ParseError):
    """Invalid operator format.

    Attributes:
        operator -- Operator text that triggered the error."""
    def __init__(self, operator):
        self.operator = operator

    def __str__(self):
        return self.operator


##############################################################################
# Constants

LESS_THAN = 0
LESS_THAN_EQUAL = 1
EQUAL = 2
GREATER_THAN_EQUAL = 3
GREATER_THAN = 4
NOT_EQUAL = 5
ALWAYS_MATCH = 6


##############################################################################
# Support functions

def text_to_operator(text):
    if not text:
        return ALWAYS_MATCH
    elif text == '<':
        return LESS_THAN
    elif text == '<=':
        return LESS_THAN_EQUAL
    elif text == '=':
        return EQUAL
    elif text == '>=':
        return GREATER_THAN_EQUAL
    elif text == '>':
        return GREATER_THAN
    elif text == '!=':
        return NOT_EQUAL
    else:
        raise BadOperatorError(text)


def operator_to_text(operator):
    if operator == ALWAYS_MATCH:
        return ' any '
    elif operator == LESS_THAN:
        return '<'
    elif operator == LESS_THAN_EQUAL:
        return '<='
    elif operator == EQUAL:
        return '='
    elif operator == GREATER_THAN_EQUAL:
        return '>='
    elif operator == GREATER_THAN:
        return '>'
    elif operator == NOT_EQUAL:
        return '!='
    else:
        raise BadOperatorError(str(operator))


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: package.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Package class for pykg-config.

Stores information read from a pkg-config file.

"""

__version__ = "$Revision: $"
# $Source$

from copy import deepcopy
from os.path import abspath, dirname, join, normpath
import re
import shlex
import sys

from pykg_config.errorprinter import ErrorPrinter
from pykg_config.exceptions import ParseError
from pykg_config.pcfile import read_pc_file
from pykg_config.substitute import substitute
from pykg_config.props import *
from pykg_config.options import Options
from pykg_config.packagespeclist import parse_package_spec_list
from pykg_config.version import BadVersionFormatError, Version

##############################################################################
# Package class

class Package:
    """This class stores the information gleaned from a pkg-config
    file, allowing quick access to it.

    """

    def __init__(self, filename=None, globals={}):
        # Different platforms may use different flags and extensions
        if sys.platform == 'win32' and Options().get_option('use_msvc_syntax'):
            self.lib_suffix = '.lib'
        else:
            self.lib_suffix = ''

        # Parse a file if one was given
        if filename is not None:
            self.load_from_pc_file(filename, globals)
            if filename.endswith('-uninstalled'):
                self.uninstalled = True
            else:
                self.uninstalled = False
        else:
            self.clear()

    def __str__(self):
        result = self.filename + '\nProperties:\n'
        for key in self.properties:
            if key == 'requires' or key == 'requires.private' or \
                    key == 'conflicts':
                result += '%s:\t%s\n' % \
                            (key, [str(a) for a in self.properties[key]])
            else:
                result += '%s:\t%s\n' % (key, self.properties[key])
        result += 'Variables:\n'
        for key in self.variables:
            result += '%s:\t%s\n' % (key, self.variables[key])
        return result

    @property
    def variables(self):
        """Variables used by the package properties."""
        return self._vars

    @variables.setter
    def variables(self, new_vars):
        self._vars = new_vars

    @property
    def properties(self):
        """Properties of the package."""
        return self._props

    @properties.setter
    def properties(self, new_props):
        self._props = new_props

    @property
    def filename(self):
        """File name of the pkg-config file this package was loaded from."""
        return self._filename

    def clear(self):
        """Clear all package data."""
        self._props = deepcopy(empty_processed_props)
        self._vars = {}
        self.raw_props = deepcopy(empty_raw_props)
        self.raw_vars = {}
        self.filename = ''

    def get_raw_property(self, prop):
        """Get a property value in its raw format, as it appears in the
        file.

        """
        return self.raw_props[prop]

    def get_raw_variable(self, var):
        """Get a variable in its raw format, as it appears in the file."""
        return self.raw_vars[var]

    def sanity_check(self):
        return True

    def load_from_pc_file(self, filename, global_variables):
        """Load data from a package config file and process it."""
        self.raw_vars, self.variables, \
                self.raw_props = read_pc_file(filename, global_variables)
        self._filename = filename
        self._process_props(global_variables)

    def _process_props(self, global_variables):
        # Processing of file data
        props = self.raw_props

        # May need to reset the prefix variable
        if sys.platform == 'win32' and \
                not Options().get_option('dont_define_prefix'):
            # Use the location of the .pc file to guess a suitable value for
            # the prefix variable. Start by checking if the absolute .pc 
            # location ends with '\lib\pkgconfig'.
            abs_loc = dirname(abspath(self.filename))
            if Options().get_option('normalise_paths'):
                abs_loc = normpath(abs_loc)
            else:
                # If not normalising paths, then all paths should be in /
                # format for consistency
                abs_loc = abs_loc.replace('\\', '/')
            if abs_loc.endswith('\\lib\\pkgconfig'):
                self.variables[Options().get_option('prefix_variable')] = \
                        abs_loc.rstrip('\\lib\\pkgconfig')
                ErrorPrinter().debug_print('Replaced {0} with \
{1}'.format(Options().get_option('prefix_variable'),
            self.variables[Options().get_option('prefix_variable')]))

        # Perform substitutions
        for key in props:
            props[key] = substitute(props[key], self.variables,
                    global_variables)

        # Parse the data
        self.properties = deepcopy(empty_processed_props)
        self.properties['name'] = props['name']
        if props['description']:
            self.properties['description'] = props['description']
        if props['version']:
            try:
                self.properties['version'] = Version(props['version'])
            except BadVersionFormatError as e:
                raise BadVersionFormatError(e.versionstring, props['name'])
        self.properties['requires'] = \
                parse_package_spec_list(props['requires'])
        self.properties['requires.private'] = \
            parse_package_spec_list(props['requires.private']) + \
            self.properties['requires']
        self.properties['conflicts'] = \
                parse_package_spec_list(props['conflicts'])
        self._parse_cflags(props['cflags'])
        self._parse_libs(props['libs'])
        self._parse_libs(props['libs.private'], dest='private.')

    def _parse_cflags(self, value):
        flags = shlex.split(value, posix=False)
        for flag in flags:
            if flag.startswith('-I'):
                if flag[2:] not in \
                        Options().get_option('forbidden_cflags'):
                    # Prepend pc_sysrootdir if necessary
                    pc_sysrootdir = Options().get_option('pc_sysrootdir')
                    if pc_sysrootdir:
                        include_dir = join(pc_sysrootdir, flag[2:].strip())
                    else:
                        include_dir = flag[2:].strip()
                    if Options().get_option('full_compatibility') and \
                            include_dir:
                        # Drop everything after the first space when trying
                        # to be fully compatible (sucky behaviour on Win32).
                        include_dir = include_dir.split()[0]
                    if sys.platform == 'win32':
                        if Options().get_option('normalise_paths'):
                            include_dir = normpath(include_dir)
                        else:
                            include_dir = include_dir.replace('\\', '/')
                    self.properties['include_dirs'].append(include_dir)
            else:
                self.properties['other_cflags'].append(flag.strip())


    def _parse_libs(self, value, dest=''):
        # Parse lib flags
        libs = shlex.split(value)
        skip_next = False
        for ii, lib in enumerate(libs):
            if skip_next:
                # Possibly skip an entry that was eaten by a -framework
                skip_next = False
                continue
            if lib.startswith('-l'):
                self.properties[dest + 'libs'].append(lib[2:].strip() + \
                        self.lib_suffix)
            elif lib.startswith('-L'):
                if lib[2:] not in \
                        Options().get_option('forbidden_libdirs'):
                    # Prepend pc_sysrootdir if necessary
                    pc_sysrootdir = Options().get_option('pc_sysrootdir')
                    if pc_sysrootdir:
                        libpath = join(pc_sysrootdir, lib[2:].strip())
                    else:
                        libpath = lib[2:].strip()
                    if Options().get_option('full_compatibility'):
                        # Drop everything after the first space when trying
                        # to be fully compatible (sucky behaviour on Win32).
                        libpath = libpath.split()[0]
                    if sys.platform == 'win32':
                        if Options().get_option('normalise_paths'):
                            libpath = normpath(libpath)
                        else:
                            libpath = libpath.replace('\\', '/')
                    self.properties[dest + 'libpaths'].append(libpath)
            elif lib.startswith('-framework'):
                self.properties[dest + 'otherlibs']. \
                        append(libs[ii + 1].strip() + self.lib_suffix)
                skip_next = True
            else:
                self.properties[dest + 'otherlibs'].append(lib.strip() + \
                        self.lib_suffix)


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: errorprinter.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Error printing singleton.

Prints strings given dependent on options set in Options(). Variables can be
set that will be replaced during string printing.

"""

__version__ = "$Revision: $"
# $Source$

from pykg_config.options import Options

class ErrorPrinter(object):
    def __new__(cls, *p, **k):
        if not '_the_instance' in cls.__dict__:
            cls._the_instance = object.__new__(cls)
        return cls._the_instance

    def set_variable(self, var, value):
        if not hasattr(self, 'vars'):
            self.vars = {}
        self.vars[var] = value

    def debug_print(self, line, args=None):
        if not Options().get_option('debug'):
            return
        if hasattr(self, 'vars'):
            for var in self.vars:
                line = line.replace('%(' + var + ')', self.vars[var])
        if args is not None:
            line = line % args
        Options().get_option('error_dest').write(line + '\n')

    def error(self, line, args=None):
        if hasattr(self, 'vars'):
            for var in self.vars:
                line = line.replace('%(' + var + ')', self.vars[var])
        if args is not None:
            line = line % args
        Options().get_option('error_dest').write(line + '\n')

    def verbose_error(self, line, args=None):
        if not Options().get_option('print_errors'):
            return
        self.error(line, args)


# vim: tw=79




#!/usr/bin/env python

# Copyright (c) 2009, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: pykg-config.py
# Author: Geoffrey Biggs
# Part of pykg-config.

__version__ = "$Revision: $"
# $Source$

from optparse import OptionParser, OptionError
from os import getenv
import sys
import traceback

from pykg_config.errorprinter import ErrorPrinter
from pykg_config.result import PkgCfgResult, NoPackagesSpecifiedError
from pykg_config.options import Options
from pykg_config.version import Version
from pykg_config.pkgsearcher import PackageNotFoundError, NoOpenableFilesError
from pykg_config.substitute import UndefinedVarError

PYKG_CONFIG_VERSION = '1.1.0'
CORRESPONDING_VERSION = '0.26'

def setup_option_parser():
    # Creates an OptionParser instance with all the options needed.
    usage = 'Usage: %prog [options] package_spec_list\n\
Return metainformation about installed libraries.'
    version = CORRESPONDING_VERSION
    parser = OptionParser(usage = usage, version = version)
    parser.add_option('--realversion', dest='realversion',
                      action='store_true', default=False,
                      help='Get the pykg-config version. [Default: %default]')
    parser.add_option('--modversion', dest='modversion', action='store_true',
                      default=False, help='Output version for package')
    parser.add_option('--atleast-pkgconfig-version',
                      dest='atleast_pkgconfig_version', type='string',
                      action='store', default='',
                      help='Require given version of pkg-config')
    parser.add_option('--libs', dest='libs', action='store_true', default=False,
                      help='Output all linker flags')
    parser.add_option('--static', dest='static', action='store_true',
                      default=False,
                      help='Output linker flags for static linking')
    parser.add_option('--short-errors', dest='short_errors',
                      action='store_true', default=False,
                      help='Print short errors')
    parser.add_option('--libs-only-l', dest='libs_only_l', action='store_true',
                      default=False, help='Output -l flags')
    parser.add_option('--libs-only-other', dest='libs_only_other',
                      action='store_true', default=False,
                      help='Output other libs (e.g. -pthread)')
    parser.add_option('--libs-only-L', dest='libs_only_big_l',
                      action='store_true', default=False,
                      help='Output -L flags')
    parser.add_option('--cflags', dest='cflags', action='store_true',
                      default=False,
                      help='Output all pre-processor and compiler flags')
    parser.add_option('--cflags-only-I', dest='cflags_only_big_i',
                      action='store_true', default=False,
                      help='Output -I flags'),
    parser.add_option('--cflags-only-other', dest='cflags_only_other',
                      action='store_true', default=False,
                      help='Output cflags not covered by --cflags-only-I')
    parser.add_option('--variable', dest='variable', type='string',
                      action='store', default=False,
                      help='Get the value of a variable'),
    parser.add_option('--define-variable', dest='define_variable',
                      type='string', action='append',
                      help='Set the value of a variable'),
    parser.add_option('--exists', dest='exists', action='store_true',
                      default=False, help='Return 0 if the module(s) exist')
    parser.add_option('--uninstalled', dest='uninstalled', action='store_true',
                      default=False,
                      help='Return 0 if the uninstalled version of one or more \
modules or their dependencies will be used')
    parser.add_option('--atleast-version', dest='atleast_version',
                      type='string', action='store', default=False,
                      help='Return 0 if the module is at least the given \
version')
    parser.add_option('--exact-version', dest='exact_version', type='string',
                      action='store', default=False,
                      help='Return 0 if the module is exactly the given \
version')
    parser.add_option('--max-version', dest='max_version', type='string',
                      action='store', default=False,
                      help='Return 0 if the module is no newer than the given \
version')
    parser.add_option('--list-all', dest='list_all', action='store_true',
                      default=False, help='List all known packages')
    parser.add_option('--debug', dest='debug', action='store_true',
                      default=False, help='Show verbose debug information')
    parser.add_option('--print-errors', dest='print_errors',
                      action='store_true', default=False,
                      help='Show verbose information about missing or \
conflicting packages')
    parser.add_option('--silence-errors', dest='silence_errors',
                      action='store_true', default=False,
                      help="Don't show verbose information about missing or \
conflicting packages")
    parser.add_option('--errors-to-stdout', dest='errors_to_stdout',
                      action='store_true', default=False,
                      help='Print errors from --print-errors to stdout')
    parser.add_option('--dump-package', dest='dump_package',
                      action='store_true', default=False,
                      help='Print the parsed package information and exit.')
    parser.add_option('--normalise-paths', dest='normalise_paths',
                      action='store_true', default=False,
                      help="Normalise paths to use the correct slash for \
your platform. [Default: %default]")
    if sys.platform == 'win32':
        parser.add_option('--dont-define-prefix', dest='dont_define_prefix',
                          action='store_true', default=False,
                          help="Don't try to override the value of prefix for \
each .pc file found with a guestimated value based on the location of the .pc \
file.")
        parser.add_option('--prefix-variable', dest='prefix_variable',
                          type='string', action='store', default='prefix',
                          help='Set the name of the variable that pkg-config \
automatically sets when defining the prefix.')
        parser.add_option('--msvc-syntax', dest='msvc_syntax',
                          action='store_true', default=False,
                          help='Output linker flags in the Microsoft compiler \
(cl) format')
        parser.add_option('--no-msvc-syntax', dest='msvc_syntax',
                          action='store_false',
                          help='Do not output linker flags in the Microsoft \
compiler (cl) format')

    # Options that change default depending on platform
    if sys.platform == 'win32':
        full_compatibility = False
    else:
        full_compatibility = True
    parser.add_option('--full-compatibility', dest='full_compatibility',
                      default=full_compatibility, action='store_true',
                      help='Enable full-compatibility mode. \
[Default: %default]')
    parser.add_option('--less-compatibility', dest='full_compatibility',
                      action='store_false',
                      help='Disable full-compatibility mode. \
[Default: %default]')

    return parser


def main(argv):
    parser = setup_option_parser()
    try:
        options, args = parser.parse_args()
    except OptionError as e:
        print('OptionError: ' + str (e))
        sys.exit(1)

    if options.realversion:
        print('{0} (Equivalent to {1}'.format(PYKG_CONFIG_VERSION,
                                              CORRESPONDING_VERSION))
        sys.exit(0)

    global_variables = {}

    zip_name = 'python{0}{1}.zip'.format(sys.version_info[0],
                                          sys.version_info[1])
    for path in sys.path:
        if path.endswith('64/' + zip_name):
            Options().set_option('is_64bit', True)
            break

    if getenv('PKG_CONFIG_SYSROOT_DIR'):
        global_variables['pc_sysrootdir'] = getenv('PKG_CONFIG_SYSROOT_DIR')
    if getenv('PKG_CONFIG_TOP_BUILD_DIR'):
        global_variables['pc_topbuilddir'] = getenv('PKG_CONFIG_TOP_BUILD_DIR')
    if getenv('PKG_CONFIG_DISABLE_UNINSTALLED'):
        Options().set_option('prefer_uninstalled', False)
    if getenv('PKG_CONFIG_ALLOW_SYSTEM_LIBS'):
        Options().set_option('forbidden_libdirs', [])
    else:
        if Options().get_option('is_64bit'):
            Options().set_option('forbidden_libdirs', ['/usr/lib64'])
        else:
            Options().set_option('forbidden_libdirs', ['/usr/lib'])
    if getenv('PKG_CONFIG_ALLOW_SYSTEM_CFLAGS'):
        Options().set_option('forbidden_cflags', [])
    else:
        forbidden = []
        if sys.platform != 'win32':
            forbidden.append('/usr/include')
        if getenv('C_INCLUDE_PATH'):
            forbidden.append(getenv('C_INCLUDE_PATH'))
        if getenv('CPLUS_INCLUDE_PATH'):
            forbidden.append(getenv('CPLUS_INCLUDE_PATH'))
        Options().set_option('forbidden_cflags', forbidden)

    if options.full_compatibility:
        Options().set_option('full_compatibility', True)
    else:
        Options().set_option('full_compatibility', False)

    if options.atleast_pkgconfig_version:
        other_version = Version(options.atleast_pkgconfig_version)
        if other_version > get_pkg_config_version():
            sys.exit(1)
        else:
            sys.exit(0)
    if options.static:
        Options().set_option('private_libs', True)
    if options.short_errors:
        Options().set_option('short_errors', True)
    if options.define_variable:
        for var_def in options.define_variable:
            sub_strings = var_def.split('=')
            if len(sub_strings) != 2:
                print('Bad argument format for define-variable: {1}'.format(var_def))
                sys.exit(1)
            global_variables[sub_strings[0]] = sub_strings[1]
    if options.debug:
        Options().set_option('debug', True)
    if options.errors_to_stdout:
        Options().set_option('error_dest', sys.stdout)
    if sys.platform == 'win32':
        if options.dont_define_prefix:
            Options().set_option('dont_define_prefix', True)
        else:
            Options().set_option('dont_define_prefix', False)
        if options.prefix_variable:
            Options().set_option('prefix_variable', options.prefix_variable)
        if options.msvc_syntax:
            Options().set_option('use_msvc_syntax', True)
        else:
            Options().set_option('use_msvc_syntax', False)
    if options.normalise_paths:
        Options().set_option('normalise_paths', True)
    else:
        Options().set_option('normalise_paths', False)

    if options.modversion or options.libs or options.libs_only_l or \
            options.libs_only_big_l or options.libs_only_other or \
            options.cflags or options.cflags_only_big_i or \
            options.cflags_only_other or options.list_all:
        if options.silence_errors:
            Options().set_option('print_errors', False)
        else:
            Options().set_option('print_errors', True)
    else:
        if options.print_errors:
            Options().set_option('print_errors', True)
        else:
            Options().set_option('print_errors', False)

    if options.list_all:
        Options().set_option('command', 'list-all')
        try:
            result = PkgCfgResult(global_variables)
            all_packages, errors = result.known_packages_list()
        except:
            ErrorPrinter().error('Exception searching for packages:')
            traceback.print_exc()
            sys.exit(1)
        if all_packages:
            max_width = max([(len(p), p) for p, n, d in all_packages])
            for package, name, description in all_packages:
                print('{0:{3}}{1} - {2}'.format(package, name, description, max_width[0] + 1))
        for e in errors:
            ErrorPrinter().error(e)
        sys.exit(0)

    try:
        Options().set_option('command', 'search')
        search = ' '.join(args)
        Options().set_option('search_string', search)
        result = PkgCfgResult(global_variables)
        result.find_packages(search, True)
    except NoOpenableFilesError as e:
        ErrorPrinter().verbose_error(str(e))
        sys.exit(1)
    except PackageNotFoundError as e:
        if not Options().get_option('short_errors'):
            ErrorPrinter().verbose_error('''Package {0} was not found in the \
pkg-config search path.
Perhaps you should add the directory containing `{0}.pc'
to the PKG_CONFIG_PATH environment variable'''.format(e.pkgname))
        ErrorPrinter().verbose_error(str(e))
        sys.exit(1)
    except NoPackagesSpecifiedError:
        Options().get_option('error_dest').write(
            'Must specify package names on the command line\n')
        sys.exit(1)
    except UndefinedVarError as e:
        ErrorPrinter().error("Variable '{0}' not defined in '{1}'".format(
            e.variable, e.pkgfile))
        sys.exit(1)
    except:
        print('Exception searching for packages')
        traceback.print_exc()
        sys.exit(1)

    if options.dump_package:
        result.dump_package()
        sys.exit(0)

    if options.exists:
        # Even if the packages don't meet the requirements, they exist, which
        # is good enough for the exists option.
        sys.exit(0)
    if options.uninstalled:
        # Check if any packages loaded (both searched-for and dependencies)
        # are uninstalled.
        if result.have_uninstalled():
            sys.exit(0)
        sys.exit(1)

    if options.modversion:
        for l in result.get_searched_pkgs_versions():
            print(l)
    found_version = \
        result.get_package_version(result.get_searched_pkg_list()[0].name)
    if options.atleast_version:
        if found_version < Version(options.atleast_version):
            sys.exit(1)
        sys.exit(0)
    if options.exact_version:
        if found_version != Version(options.exact_version):
            sys.exit(1)
        sys.exit(0)
    if options.max_version:
        if found_version > Version(options.max_version):
            sys.exit(1)
        sys.exit(0)

    if options.variable:
        value = result.get_variable_value(options.variable)
        if value == None:
            print('')
        else:
            print(value)
    if options.cflags_only_big_i:
        print(result.get_big_i_flags())
    if options.cflags_only_other:
        print(result.get_other_i_flags())
    if options.cflags:
        print(result.get_cflags())
    if options.libs_only_l:
        print(result.get_l_flags())
    if options.libs_only_big_l:
        print(result.get_big_l_flags())
    if options.libs_only_other:
        print(result.get_other_l_flags())
    if options.libs:
        print(result.get_all_lib_flags())


def get_pkg_config_version():
    return Version(CORRESPONDING_VERSION)


if __name__ == '__main__':
    main(sys.argv)

# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: version.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Version object.

Stores a version and can compare it with other versions.

"""

__version__ = "$Revision: $"
# $Source$

import re

from pykg_config.errorprinter import ErrorPrinter
from pykg_config.exceptions import ParseError

##############################################################################
# Exceptions

class BadVersionFormatError(ParseError):
    """Generic error parsing a pkg-config file.

    Attributes:
        versionstring -- String containing the badly-formatted version."""
    def __init__(self, versionstring, package = ''):
        self.versionstring = versionstring
        self.package = package

    def __str__(self):
        return '%s: %s' % (self.package, self.versionstring)


##############################################################################
# Version class

class Version:
    def __init__(self, version_string=None):
        if version_string is not None:
            self._parse_version(version_string)
            ErrorPrinter().debug_print('Parsed %s into %s',
                                       (version_string, self.comps))
        else:
            self.raw_string = '0'
            self.comps = [0]

    def __str__(self):
        return self.raw_string

    def __lt__(self, other):
        if self._compare_components(other.comps) == -1:
            return True
        return False

    def __le__(self, other):
        result = self._compare_components(other.comps)
        if result == -1 or result == 0:
            return True
        return False

    def __eq__(self, other):
        if self._compare_components(other.comps) == 0:
            return True
        return False

    def __ne__(self, other):
        if self._compare_components(other.comps) != 0:
            return True
        return False

    def __gt__(self, other):
        if self._compare_components(other.comps) == 1:
            return True
        return False

    def __ge__(self, other):
        result = self._compare_components(other.comps)
        if result == 0 or result == 1:
            return True
        return False

    def is_empty(self):
        if self == Version():
            return True
        return False

    def _parse_version(self, version_string):
        # Parse a version into components.
        self.raw_string = version_string
        self.comps = []
        start = 0
        while start < len(version_string):
            m = re.match(r'[-._~+ ]?(?P<comp>[a-zA-Z0-9%]+)',
                         version_string[start:], re.U)
            if m is None:
                # Stop pulling out components when the start of the string
                # no longer matches
                #raise BadVersionFormatError(version_string)
                # pkg-config apparently ignores poorly-formatted versions
                return
            comp = m.group('comp')
            try:
                comp = int(comp)
            except ValueError:
                pass
            self.comps.append(comp)
            start += m.end()

    def _compare_components(self, other_comps):
        # Loop through the components, comparing each one in turn.
        # When one differs, return how it differs.
        # -1 for less than, 0 for equal, 1 for greater than.
        if self.comps == other_comps:
            return 0
        comp_len = min(len(self.comps), len(other_comps))
        for left, right in zip(self.comps[:comp_len], other_comps[:comp_len]):
            if left < right:
                return -1
            elif left > right:
                return 1
            else:
                continue
        # Have exhausted all comps in the minimal list, so now see if
        # the trailing part of the longer list is all 0 or something else.
        # All 0 means it's still equal, something else means it's greater
        # than.
        if len(self.comps) < len(other_comps):
            remainder = other_comps[comp_len:]
            # If other_comps remainder is not all 0, self is less than.
            result = -1
        elif len(self.comps) == len(other_comps):
            # There is no left over bit, versions must be equal
            return 0
        else:
            remainder = self.comps[comp_len:]
            result = 1
        for comp in remainder:
            try:
                comp = int(comp)
                if comp != 0:
                    return result
            except ValueError:
                return result
        # If got to here, then none of the trailing components were non-zero,
        # so they don't matter and the final result is equal.
        return 0


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: pcfile.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Parse pkg-config files.

Contains functions that read and parse metadata files in the format
used by pkg-config.

"""

__version__ = "$Revision: $"
# $Source$

import re

from pykg_config.errorprinter import ErrorPrinter
from pykg_config.exceptions import ParseError
from pykg_config.substitute import substitute
from pykg_config.props import empty_raw_props

# Constants
VARIABLE = 0
PROPERTY = 1
empty_vars = {}

##############################################################################
# Exceptions

class EmptyPackageFileError(ParseError):
    """The given pkg-config file had no lines in it."""
    pass


class MalformedLineError(ParseError):
    """The line is not a correctly-formatted variable or a property.

    Attributes:
        line -- The incorrectly-formatted line.

    """
    def __init__(self, line):
        self.line = line

    def __str__(self):
        return self.line


class MultiplyDefinedValueError(ParseError):
    """A value has been defined more than once.

    Attributes:
        line -- The line containing the duplicate value.

    """
    def __init__(self, line):
        self.line = line

    def __str__(self):
        return self.line


class TrailingContinuationCharError(ParseError):
    """The last line in a file has a trailing continuation character.

    Attributes:
        line -- The line containing the trailing character.

    """
    def __init__(self, line):
        self.line = line

    def __str__(self):
        return self.line


##############################################################################
# Public functions

def read_pc_file(filename, global_variables):
    """Read and parse it into two dictionaries (variables and properties).

    Returns variables and properties.

    """
    ErrorPrinter().set_variable('filename', filename)
    ErrorPrinter().debug_print('Parsing %(filename)')
    pcfile = open(filename, 'r')
    lines = pcfile.readlines()
    if not lines:
        raise EmptyPackageFileError(filename)
    raw_vars, vars, props = parse_pc_file_lines(lines, global_variables)
    pcfile.close()
    return raw_vars, vars, props


##############################################################################
# Private functions

def parse_pc_file_lines(lines, globals):
    # Parse all lines from a pkg-config file, building vars and props
    # dictionaries.
    raw_vars = {}
    vars = {}
    props = empty_raw_props.copy()
    seen_props = []
    for line in merge_lines(lines, '\\'):
        raw_vars, vars, props, seen_props = parse_line(strip_comments(line).strip(),
                                                       raw_vars, vars, props,
                                                       seen_props, globals)

    return raw_vars, vars, props


def merge_lines(lines, cont_char):
    # Merge any lines ending with the given character with the following line.
    # Return a list of lines. Raises TrailingContinuationCharError if the
    # final line has the continuation character.
    if lines[-1][-1] == cont_char:
        raise TrailingContinuationCharError(line)
    result = []
    ii = 0
    while ii < len(lines):
        new_line = lines[ii].rstrip()
        if new_line == '':
            ii += 1
            continue
        while new_line[-1] == cont_char:
            # Drop the \n and the continuation char
            new_line = new_line[:-2] + ' '
            ii += 1
            new_line += lines[ii].rstrip()
        result.append(new_line)
        ii += 1
    return result


def parse_line(line, raw_vars, vars, props, seen_props, globals):
    # Parse a single line from the file, adding its value to the props or vars
    # dictionary as appropriate.
    if not line:
        return raw_vars, vars, props, seen_props
    key, value, type = split_pc_file_line(line)
    # Check first if it's one of the known keys.
    if type == VARIABLE:
        # Perform substitution using variables found so far and global
        # variables, then store the result.
        if key in vars:
            raise MultiplyDefinedValueError(key)
        if key in globals:
            ErrorPrinter().debug_print('Adding %s -> %s to vars from globals',
                                       (key, value))
            raw_vars[key] = value.strip ()
            vars[key] = substitute (globals[key], vars, globals)
        else:
            ErrorPrinter().debug_print('Adding %s -> %s to vars', (key, value))
            raw_vars[key] = value.strip ()
            vars[key] = substitute (value.strip(), vars, globals)
    elif type == PROPERTY:
        if key in seen_props:
            raise MultiplyDefinedValueError(key)
        if key.lower() in empty_raw_props:
            if value is None:
                value = empty_raw_props[key.lower()]
            ErrorPrinter().debug_print('Adding %s -> %s to props', (key, value))
            props[key.lower ()] = value
            seen_props.append(key)
        else:
            # As per the original pkg-config, don't raise errors on unknown
            # keys because they may be from future additions to the file
            # format. But log an error
            ErrorPrinter().debug_print('Unknown key/value in %(filename)s:\n%s: %s',
                                       (key, value))
    else:
        # Probably a malformed line. Ignore it.
        pass
    return raw_vars, vars, props, seen_props


def strip_comments(line):
    # Strip comments from a line, returning the uncommented part or a blank
    # string if the whole line was a comment.
    commentStart = line.find('#')
    if commentStart == -1:
        return line
    else:
        return line[:commentStart]


property_re = re.compile('(?P<key>[\w.]+):\s*(?P<value>.+)?', re.U)
variable_re = re.compile('(?P<var>[\w.]+)=\s*(?P<value>.+)?', re.U)
def split_pc_file_line(line):
    # Split a line into key and value, and determine if it is a property or a
    # variable.
    m = property_re.match(line)
    if m is not None:
        return m.group('key'), m.group('value'), PROPERTY

    m = variable_re.match(line)
    if m is not None:
        if m.group('value') is None:
            return m.group('var'), '', VARIABLE
        else:
            return m.group('var'), m.group('value'), VARIABLE

    # Gloss over malformed lines (that's what pkg-config does).
    ErrorPrinter().debug_print('Malformed line: {0}'.format(line))
    return None, None, None


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: packagespeclist.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Parses a textual list of packages with optional version constraints.

"""

__version__ = "$Revision: $"
# $Source$

import re

from pykg_config.version import Version
from pykg_config.dependency import *

def parse_package_spec_list(value):
    """Parses a textual list of package specs into a list of Dependency
    objects containing name, and possibly a version restriction.

    """
    result = []
    matches = re.findall('(?P<name>[^\s,!=<>]+)(,|\s*(?P<operator>[!=<>]+)\s*(?P<version>[^\s,]+))?',
               value.strip(), re.U)

    for package in matches:
        name = package[0]
        operator = text_to_operator(package[2])
        if package[3]:
            version = Version(package[3])
        else:
            version = Version()
        result.append (Dependency(name, operator, version))
    return result


# vim: tw=79




#!/usr/bin/env python

# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: test_unittests.py
# Author: Geoffrey Biggs
# Part of pykg-config.

__version__ = "$Revision: $"
# $Source$

import os
import re
import subprocess
import unittest

from pykg_config import packagespeclist
from pykg_config import substitute
from pykg_config import dependency
from pykg_config import version


def call_process(args):
    env = os.environ
    env['PKG_CONFIG_PATH'] = '{0}:{1}'.format(os.getcwd(),
            env['PKG_CONFIG_PATH'])
    process = subprocess.Popen(args, stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE)
    output = process.communicate()
    output = (output[0].strip(), output[1].strip())
    return_code = process.returncode
    return output[0], output[1], return_code


class TestVersion(unittest.TestCase):
    def setUp(self):
        self.strings = ['2.3', '5.0-svn', '1.2.3.4.a.6', '15', '2.3.0',
                        '2.3.1', '2.3-a']
        self.parsed = [[2, 3], [5, 0, 'svn'], [1, 2, 3, 4, 'a', 6], [15],
                       [2, 3, 0], [2, 3, 1], [2, 3, 'a']]

    def test_constructor(self):
        for case, result in zip(self.strings, self.parsed):
            self.assertEqual(version.Version(case).comps, result)

    def test_comparisons(self):
        versions = []
        for verstring in self.strings:
            versions.append(version.Version(verstring))
        self.assert_(versions[0] < versions[1])
        self.assert_(versions[0] < versions[5])
        self.assert_(versions[0] < versions[6])
        self.assert_(versions[0] <= versions[0])
        self.assert_(versions[0] <= versions[1])
        self.assert_(versions[0] <= versions[4])
        self.assert_(versions[0] == versions[0])
        self.assert_(versions[0] == versions[4])
        self.assert_(versions[1] == versions[1])
        self.assert_(versions[0] != versions[1])
        self.assert_(versions[0] != versions[2])
        self.assert_(versions[0] != versions[3])
        self.assert_(versions[0] > versions[2])
        self.assert_(versions[0] >= versions[0])
        self.assert_(versions[0] >= versions[2])
        self.assert_(versions[0] >= versions[4])

        self.assertFalse(versions[0] < versions[2])
        self.assertFalse(versions[0] <= versions[2])
        self.assertFalse(versions[0] == versions[1])
        self.assertFalse(versions[0] == versions[2])
        self.assertFalse(versions[0] == versions[3])
        self.assertFalse(versions[0] != versions[4])
        self.assertFalse(versions[0] >= versions[1])
        self.assertFalse(versions[0] >= versions[3])
        self.assertFalse(versions[0] >= versions[5])
        self.assertFalse(versions[0] >= versions[6])
        self.assertFalse(versions[0] > versions[1])
        self.assertFalse(versions[0] > versions[3])

class TestPackage(unittest.TestCase):
    def test_parse_package_spec_list(self):
        parsed = packagespeclist.parse_package_spec_list('blag < 2.3, \
                blerg>=5-svn blork, bleck = 15')
        expected = [dependency.Dependency('blag', dependency.LESS_THAN,
                                          version.Version('2.3')),
                    dependency.Dependency('blerg', dependency.GREATER_THAN_EQUAL,
                                          version.Version('5-svn')),
                    dependency.Dependency('blork', dependency.ALWAYS_MATCH,
                                          version.Version()),
                    dependency.Dependency('bleck', dependency.EQUAL,
                                          version.Version('15'))]
        self.assertEqual(parsed, expected)


class TestSubstitutions(unittest.TestCase):
    def setUp(self):
        self.pykg_config_cmd = '../pykg-config.py'
        self.vars = {'blag1': 'not recursive',
                     'blag2': 'references ${blag1} variable',
                     'blag3': 'recursive with ${blag3}'}

    def test_escaping(self):
        value = 'a $${blag1} that should not be replaced'
        result = 'a ${blag1} that should not be replaced'
        self.assertEqual(substitute.substitute(value, self.vars), result)

    def test_substitute(self):
        value = 'stuff ${blag2} more stuff ${blag1} ${blag3}'
        result = 'stuff references not recursive variable more stuff not recursive recursive with ${blag3}'
        self.assertEqual(substitute.substitute(value, self.vars), result)

    def test_get_to_replace_re(self):
        self.assertEqual(substitute.get_to_replace_re('blag'),
                         re.compile ('(?<!\$)\$\{blag\}', re.U))

    def test_get_all_substitutions(self):
        nameful = '${lots} of ${names} ${reffed}.'
        self.assertEqual(substitute.get_all_substitutions(nameful),
                         ['lots', 'names', 'reffed'])

    def test_undefined_var(self):
        args = ['--cflags', 'global_var']
        stdout, stderr, ret_code = call_process([self.pykg_config_cmd] + args)
        self.assertEqual(ret_code, 1)

    def test_global_var(self):
        args = ['--cflags', 'global_var', '--define-variable=global_var=blurg']
        stdout, stderr, ret_code = call_process([self.pykg_config_cmd] + args)
        self.assertEqual(stdout, '-Iblurg')
        self.assertEqual(ret_code, 0)


class TestQuoteEscapes(unittest.TestCase):
    def setUp(self):
        self.pykg_config_cmd = '../pykg-config.py'

    def test_maintain_escaping(self):
        args = ['--cflags', 'unittests1']
        stdout, stderr, ret_code = call_process([self.pykg_config_cmd] + args)
        self.assertEqual(stdout, r'-DPATH=\"/opt/shaders/\"')
        self.assertEqual(ret_code, 0)



if __name__ == '__main__':
    unittest.main()


# vim: tw=79




#!/usr/bin/env python

# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

__version__ = "$Revision: $"
# $Source$

from distutils.core import setup
from distutils.command.build_py import build_py
from distutils import log
import os.path
import sys

scripts = ['pykg-config.py']
if sys.platform == 'win32':
    scripts.append('pkg-config.bat')


class BuildWithConfigure(build_py):
    user_options = build_py.user_options + [
        ('with-pc-path=', None, 'default search path for .pc files'),
        ]

    def __init__(self, dist):
        build_py.__init__(self, dist)
        self.with_pc_path = None

    def finalize_options(self):
        build_py.finalize_options(self)
        self.ensure_dirname('with_pc_path')

    def run(self):
        build_py.run(self)
        config_dest = os.path.join(self.build_lib, 'pykg_config', 'install_config.py')
        log.info("creating configuration file at %s", config_dest)
        with open(config_dest, 'w') as f:
            if self.with_pc_path:
                f.write('pc_path = "' + self.with_pc_path + '"\n')
            else:
                f.write('pc_path = None\n')
        # TODO: should byte-compile the config file here (see
        # build_py.byte_compile())


setup(name='pykg-config',
      version='1.3.0',
      description='pkg-config replacement.',
      author='Geoffrey Biggs',
      author_email='git@killbots.net',
      url='http://github.com/gbiggs/pykg-config',
      license='BSD',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Topic :: Software Development',
          'Topic :: Utilities'
          ],
      packages=['pykg_config'],
      scripts=scripts,
      cmdclass={'build_py':BuildWithConfigure}
      )



#!/usr/bin/env python

# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: test_compatibility.py
# Author: Geoffrey Biggs
# Part of pykg-config.

__version__ = "$Revision: $"
# $Source$

# TODO: Selecting random packages from the list for the tests is a bad idea. It
# doesn't ensure complete repeatability. Instead, the tests should run for each
# package in the list.

import random
import subprocess
import sys
import unittest

class TestCompatibility(unittest.TestCase):
    def setUp(self):
        self.packages = [('dbus-1', '1.4.16'),
                         ('openssl', '1.0.0g'),
                         ('libxml-2.0', '2.7.8'),
                         ('QtCore', '4.7.4')]
        self.error_package = 'thisisabadpkg'
        self.pykg_config_command = '../pykg-config.py'
        self.pkg_config_command = 'pkg-config'

    def get_random_package(self):
        return random.choice(self.packages)[0]

    def get_random_package_with_version(self):
        return random.choice(self.packages)

    def get_random_package_with_incremented_version(self):
        pkg = self.get_random_package_with_version()
        # Bump the version number of the chosen package
        version = pkg[1].split('.')
        version = [version[0], str(int(version[1]) + 1)] + version[2:]
        return (pkg[0], '.'.join(version))

    def get_random_package_with_decremented_version(self):
        pkg = self.get_random_package_with_version()
        # Drop the version number of the chosen package
        version = pkg[1].split('.')
        if int(version[1]) == 0:
            version = [str(int(version[0]) - 1)] + version[2:]
        else:
            version = [version[0], str(int(version[1]) - 1)] + version[2:]
        return (pkg[0], '.'.join(version))

    def call_process(self, args):
        process = subprocess.Popen(args, stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        output = process.communicate()
        output = (output[0].strip(), output[1].strip())
        return_code = process.returncode
        return output[0], output[1], return_code

    def run_test_case(self, args):
        lhs_result = self.call_process([self.pykg_config_command] + args)
        rhs_result = self.call_process([self.pkg_config_command] + args)
        return lhs_result, rhs_result

    def build_error_msg(self, args, lhs, rhs):
        return 'Failed with arguments: {0}\nLHS: {1}\nRHS: {2}'.format(args,
                                                                       lhs,
                                                                       rhs)

    def test_noargs(self):
        lhs, rhs = self.run_test_case([])
        self.assertEqual(lhs, rhs)

    def test_version(self):
        args = ['--version']
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs)

    def test_atleast_pkgconfig_greater(self):
        args = ['--atleast-pkgconfig-version=1.0']
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs)

    def test_atleast_pkgconfig_equal(self):
        args = ['--atleast-pkgconfig-version=0.23']
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs)

    def test_atleast_pkgconfig_less(self):
        args = ['--atleast-pkgconfig-version=0.1']
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs)

    def test_modversion_existing_package(self):
        pkg = self.get_random_package_with_version()
        args = ['--modversion', pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[0], pkg[1], self.build_error_msg(args, lhs[0], pkg[1]))

    def test_modversion_nonexisting_package(self):
        pkg = self.get_random_package_with_version()
        args = ['--modversion', pkg[0] + 'blag']
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 1, self.build_error_msg(args, lhs, rhs))

    def test_modversion_no_package(self):
        args = ['--modversion']
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 1, self.build_error_msg(args, lhs, rhs))

    def test_atleast_modversion_greater(self):
        pkg = self.get_random_package_with_incremented_version()
        args = ['--atleast-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 1, self.build_error_msg(args, lhs, rhs))

    def test_atleast_modversion_equal(self):
        pkg = self.get_random_package_with_version()
        args = ['--atleast-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs[2], '0'))

    def test_atleast_modversion_less(self):
        pkg = self.get_random_package_with_decremented_version()
        args = ['--atleast-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_exact_modversion_greater(self):
        pkg = self.get_random_package_with_incremented_version()
        args = ['--exact-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 1, self.build_error_msg(args, lhs, rhs))

    def test_exact_modversion_equal(self):
        pkg = self.get_random_package_with_version()
        args = ['--exact-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_exact_modversion_less(self):
        pkg = self.get_random_package_with_decremented_version()
        args = ['--exact-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 1, self.build_error_msg(args, lhs, rhs))

    def test_max_modversion_greater(self):
        pkg = self.get_random_package_with_incremented_version()
        args = ['--max-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_max_modversion_equal(self):
        pkg = self.get_random_package_with_version()
        args = ['--max-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_max_modversion_less(self):
        pkg = self.get_random_package_with_decremented_version()
        args = ['--max-version', pkg[1], pkg[0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 1, self.build_error_msg(args, lhs[2], '1'))

    def test_exists_existing(self):
        pkg = self.get_random_package()
        args = ['--exists', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_exists_nonexisting(self):
        pkg = self.get_random_package()
        args = ['--exists', pkg + 'blag']
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 1, self.build_error_msg(args, lhs, rhs))

    def test_exists_with_correct_version(self):
        pkg = self.get_random_package_with_version()
        args = ['--exists', pkg[0], '=', pkg[1]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_exists_with_incorrect_version(self):
        pkg = self.get_random_package_with_incremented_version()
        args = ['--exists', pkg[0], '=', pkg[1]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 1, self.build_error_msg(args, lhs, rhs))

    def test_get_variable_existing(self):
        args = ['--variable', 'prefix', self.packages[0][0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_get_variable_nonexisting(self):
        args = ['--variable', 'prefix', self.packages[1][0]]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_define_variable(self):
        pkg = self.get_random_package()
        args = ['--define-variable', 'prefix=/usr/local', '--cflags', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))
        self.assertEqual(lhs[2], 0, self.build_error_msg(args, lhs, rhs))

    def test_uninstalled(self):
        pkg = self.get_random_package()
        args = ['--uninstalled', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_list_all(self):
        # The ordering of the list-all output is different between the two
        # programs due to the two different methods used for managing the list
        # of known packages and their order of precedence.
        args = ['--list-all']
        lhs, rhs = self.run_test_case(args)
        lhs_pkgs = lhs[0].split('\n')
        rhs_pkgs = rhs[0].split('\n')
        for lhs_pkg in lhs_pkgs:
            self.assert_(lhs_pkg in rhs_pkgs, '"{0}" not in RHS'.format(lhs_pkg))
        for rhs_pkg in rhs_pkgs:
            self.assert_(rhs_pkg in lhs_pkgs, '"{0}" not in LHS'.format(rhs_pkg))
        self.assertEqual(len(lhs_pkgs), len(rhs_pkgs))
        self.assertEqual(lhs[1], rhs[1], self.build_error_msg(args, lhs[1], rhs[1]))
        self.assertEqual(lhs[2], rhs[2], self.build_error_msg(args, lhs[2], rhs[2]))

    def test_print_errors_with_error(self):
        # This test relies on there being an error present. I have a dead
        # symlink in one of the searched dirs to cause an error (no such file
        # or dir) when performing the list-all command.
        args = ['--cflags', '--print-errors', self.error_package]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_print_errors_without_error(self):
        # This test relies on using a command that does _not_ produce any
        # errors.
        pkg = self.get_random_package()
        args = ['--print-errors', '--modversion', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_silence_errors_with_error(self):
        # This test relies on there being an error present. I have a dead
        # symlink in one of the searched dirs to cause an error (no such file
        # or dir) when performing the list-all command.
        args = ['--cflags', '--silence-errors', self.error_package]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_silence_errors_without_error(self):
        # This test relies on using a command that does _not_ produce any
        # errors.
        pkg = self.get_random_package()
        args = ['--silence-errors', '--modversion', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_errors_to_stdout(self):
        args = ['--cflags', '--errors-to-stdout', self.error_package]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_short_errors(self):
        args = ['--cflags', '--short-errors', 'nonexistentpackage']
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_libs(self):
        pkg = self.get_random_package()
        args = ['--libs', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_static(self):
        pkg = self.get_random_package()
        args = ['--libs', '--static', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_libs_only_l(self):
        pkg = self.get_random_package()
        args = ['--libs-only-l', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_libs_only_other(self):
        pkg = self.get_random_package()
        args = ['--libs-only-other', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_libs_only_big_l(self):
        pkg = self.get_random_package()
        args = ['--libs-only-L', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_cflags(self):
        pkg = self.get_random_package()
        args = ['--cflags', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_cflags_only_big_i(self):
        pkg = self.get_random_package()
        args = ['--cflags-only-I', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))

    def test_cflags_only_other(self):
        pkg = self.get_random_package()
        args = ['--cflags-only-other', pkg]
        lhs, rhs = self.run_test_case(args)
        self.assertEqual(lhs, rhs, self.build_error_msg(args, lhs, rhs))


if __name__ == '__main__':
    unittest.main()




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: pkgsearcher.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Searches for a pkg-config file matching a given specification.

"""

__version__ = "$Revision: $"
# $Source$

from os import getenv, listdir
from os.path import isdir, isfile, join, split, splitext
import sys
if sys.platform == 'win32':
    import _winreg

from pykg_config.exceptions import PykgConfigError
from pykg_config.options import Options
from pykg_config.errorprinter import ErrorPrinter
from pykg_config.package import Package
from pykg_config.substitute import UndefinedVarError

try:
    from pykg_config.install_config import pc_path
except ImportError:
    # If the install_config module is not available (which is the case when
    # running from the source instead of an installed version), use defaults
    pc_path = None

##############################################################################
# Exceptions

class PackageNotFoundError(PykgConfigError):
    """A .pc file matching the given package name could not be found.

    Attributes:
        pkgname -- The name of the package that could not be found.

    """
    def __init__(self, pkgname):
        self.pkgname = pkgname

    def __str__(self):
        return "No package '{0}' found".format(self.pkgname)


class NoOpenableFilesError(PackageNotFoundError):
    pass


class BadPathError(PykgConfigError):
    """A specified path is bad in some way.

    Attributes:
        path - The bad path.

    """
    def __init__(self, path):
        self.path = path

    def __str__(self):
        return 'Bad path: {0}'.format(self.path)


class NotAFileError(BadPathError):
    pass


class NotAPCFileError(BadPathError):
    pass


##############################################################################
# PkgSearcher object

class PkgSearcher:
    def __init__(self):
        # This is a dictionary of packages found in the search path. Each
        # package name is linked to a list of full paths to .pc files, in
        # order of priority. Earlier in the list is preferred over later.
        self._known_pkgs = {}

        self._init_search_dirs()

    def search_for_package(self, dep, globals):
        """Search for a package matching the given dependency specification
        (name and version restriction). Raise PackageNotFoundError if no
        matching package is found.

        Returns a parsed package object.

        """
        # Get a list of pc files matching the package name
        if isfile(dep.name) and splitext(dep.name)[1] == '.pc':
            # No need to search for a pc file
            ErrorPrinter().debug_print('Using provided pc file %s', (dep.name))
            pcfiles = [dep.name]
        else:
            ErrorPrinter().debug_print('Searching for package matching %s', (dep))
            pcfiles = self.search_for_pcfile(dep.name)
        ErrorPrinter().debug_print('Found .pc files: %s', (str(pcfiles)))
        if not pcfiles:
            raise PackageNotFoundError(str(dep))
        # Filter the list by those files that meet the version specification
        pkgs = []
        for pcfile in pcfiles:
            try:
                pkgs.append(Package(pcfile, globals))
            except IOError as e:
                ErrorPrinter().verbose_error("Failed to open '{0}': \
{1}".format(pcfile, e.strerror))
                continue
            except UndefinedVarError as e:
                raise UndefinedVarError(e.variable, pcfile)
        if not pkgs and pcfiles:
            # Raise an error indicating that all pc files we could try were
            # unopenable. This is necessary to match pkg-config's odd lack of
            # the standard "Package not found" error when a bad file is
            # encountred.
            raise NoOpenableFilesError(str(dep))
        pkgs = [pkg for pkg in pkgs \
                if dep.meets_requirement(pkg.properties['version'])]
        ErrorPrinter().debug_print('Filtered to %s',
                                   ([pkg.properties['name'] for pkg in pkgs]))
        if not pkgs:
            raise PackageNotFoundError(str(dep))
        return pkgs[0]

    def search_for_pcfile(self, pkgname):
        """Search for one or more pkg-config files matching the given
        package name. If a matching pkg-config file cannot be found,
        an empty list will be returned.

        The dictionary of known packages is stored in _known_pkgs and is
        initialised by calling init_search_dirs().

        """
        ErrorPrinter().debug_print('Looking for files matching %s', (pkgname))
        if Options().get_option('prefer_uninstalled'):
            if pkgname + '-uninstalled' in self._known_pkgs:
                # Prefer uninstalled version of a package
                ErrorPrinter().debug_print('Using uninstalled package %s',
                                           (self._known_pkgs[pkgname + '-uninstalled']))
                return self._known_pkgs[pkgname + '-uninstalled']
            elif Options().get_option('uninstalled_only'):
                ErrorPrinter().debug_print('Uninstalled only, no suitable package.')
                return []
        if pkgname in self._known_pkgs:
            ErrorPrinter().debug_print('Using any package: %s',
                             (self._known_pkgs[pkgname]))
            return self._known_pkgs[pkgname]
        else:
            ErrorPrinter().debug_print('No suitable package found')
            return []

    def known_packages_list(self):
        """Return a list of all packages found on the system, giving a name and
        a description (from the .pc file) for each, and also a list of any
        errors encountered.

        """
        result = []
        errors = []
        for pkgname in self._known_pkgs:
            # Use the highest-priority version of the package
            try:
                pkg = Package(self._known_pkgs[pkgname][0])
            except IOError as e:
                ErrorPrinter().verbose_error("Failed to open '{0}': \
{1}".format(self._known_pkgs[pkgname][0], e.strerror))
                continue
            except UndefinedVarError as e:
                errors.append("Variable '{0}' not defined in '{1}'".format(e,
                    self._known_pkgs[pkgname][0]))
                continue
            result.append((pkgname, pkg.properties['name'], pkg.properties['description']))
        return result, errors

    def _init_search_dirs(self):
        # If a hard-coded path has been set, use that _only_
        if pc_path:
            for d in pc_path.split(self._split_char()):
                self._append_packages(d)
            return
        # Otherwise use some suitable defaults
        # Append dirs in PKG_CONFIG_PATH
        if getenv('PKG_CONFIG_PATH'):
            for d in getenv('PKG_CONFIG_PATH').split(self._split_char()):
                if not d or not isdir(d):
                    continue
                self._append_packages(d)
        # Append dirs in PKG_CONFIG_LIBDIR
        if getenv('PKG_CONFIG_LIBDIR'):
            for d in getenv('PKG_CONFIG_LIBDIR').split(self._split_char()):
                if not d or not isdir(d):
                    continue
                self._append_packages(d)
        # Else append prefix/lib/pkgconfig, prefix/share/pkgconfig
        else:
            if Options().get_option('is_64bit'):
                suffix = '64'
            else:
                suffix = ''
            if isdir(join(sys.prefix, 'lib' + suffix + '/pkgconfig')):
                self._append_packages(join(sys.prefix,
                                           'lib' + suffix + '/pkgconfig'))
            if isdir(join(sys.prefix, 'share/pkgconfig')):
                self._append_packages(join(sys.prefix, 'share/pkgconfig'))
        if sys.platform == 'win32':
            key_path = 'Software\\pkg-config\\PKG_CONFIG_PATH'
            for root in ((_winreg.HKEY_CURRENT_USER, 'HKEY_CURRENT_USER'),
                         (_winreg.HKEY_LOCAL_MACHINE, 'HKEY_LOCAL_MACHINE')):
                try:
                    key = _winreg.OpenKey(root[0], key_path)
                except WindowsError as e:
                    ErrorPrinter().debug_print('Failed to add paths from \
{0}\\{1}: {2}'.format(root[1], key_path, e))
                    continue
                try:
                    num_subkeys, num_vals, modified = _winreg.QueryInfoKey(key)
                    for ii in range(num_vals):
                        name, val, type = _winreg.EnumValue(key, ii)
                        if type == _winreg.REG_SZ:
                            self._append_packages(val)
                except WindowsError as e:
                    ErrorPrinter().debug_print('Failed to add paths from \
{0}\\{1}: {2}'.format(root[1], key_path, e))
                finally:
                    _winreg.CloseKey(key)

    def _append_packages(self, d):
        ErrorPrinter().debug_print('Adding .pc files from %s to known packages',
                                   (d))
        files = listdir(d)
        for filename in files:
            if filename.endswith('.pc'):
                # Test if the file can be opened (pkg-config glosses over,
                # e.g. links that are now dead, as if they were never there).
                full_path = join(d, filename)
                name = filename[:-3]
                if name in self._known_pkgs:
                    if full_path not in self._known_pkgs[name]:
                        self._known_pkgs[name].append(full_path)
                        ErrorPrinter().debug_print('Package %s has a duplicate file: %s',
                                                   (name, self._known_pkgs[name]))
                else:
                    self._known_pkgs[name] = [full_path]

    def _split_char(self):
        # Get the character used to split a list of directories.
        if sys.platform == 'win32':
            return ';'
        return ':'

    def _can_open_file(self, filename):
        try:
            result = open(filename, 'r')
        except IOError as e:
            ErrorPrinter().debug_print('Could not open {0}'.format(filename))
            search_string = Options().get_option('search_string').split()
            if (not search_string and \
                    Options().get_option('command') == 'list-all') or \
                    True in [p.startswith(split(filename)[-1].split('.')[0]) \
                                for p in search_string]:
                ErrorPrinter().verbose_error("Failed to open '{0}': {1}".format(filename,
                                                                                e.strerror))
            return False
        return True


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: substitute.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Substitutes values. 

Recursive substitution can be performed until no substitutions are left.
An infinite recursion check is performed first to prevent infinite
loops in this case.

"""

__version__ = "$Revision: $"
# $Source$

import re

from pykg_config.exceptions import PykgConfigError

##############################################################################
# Exceptions

class InfiniteRecursionError(PykgConfigError):
    """A variable refers to itself, which will cause infinite recursion
    when substitution is performed.

    Attributes:
        variable -- The variable that refers to itself.

    """
    def __init__(self, variable):
        self.variable = variable

    def __str__(self):
        return self.variable


class UndefinedVarError(PykgConfigError):
    """A variable is not defined.

    Attributes:
        variable -- The variable that is not defined.
        pkgfile -- The file with the error. Not always set.

    """
    def __init__(self, variable, pkgfile=None):
        self.variable = variable
        self.pkgfile = pkgfile

    def __str__(self):
        return self.variable


##############################################################################
# Public functions

def substitute(value, replacements, globals={}):
    """Substitutes variables once.

    Variables in the given value are substituted. No recursion is
    performed. replacements is a dictionary of variables.

    """
    for name in get_all_substitutions(value):
        if name in globals:
            value = replace_in_string(value, name, globals[name])
        elif name in replacements:
            value = replace_in_string(value, name, replacements[name])
        else:
            raise UndefinedVarError(name)
    value = collapse_escapes(value)
    return value


##############################################################################
# Private functions

def replace_in_string(value, name, substitution):
    # Replace all instances of name in value with substitution
    to_replace = get_to_replace_re(name)
    return to_replace.sub(substitution, value)


def get_to_replace_re(name):
    # Build the re object that matches a given substition name
    return re.compile('(?<!\$)\$\{%s\}' % name, re.U)


def get_all_substitutions(value):
    found_names = re.findall('(?<!\$)\${(?P<name>[\w.]+)\}', value, flags=re.U)
    names = []
    for name in found_names:
        if name not in names:
            names.append (name)
    return names


def collapse_escapes(value):
    return value.replace('$$', '$')


# vim: tw=79




# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: props.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Defines the empty property dictionaries.

These are defined here to keep them together for ease of maintenance
when new properties are added.

"""

__version__ = "$Revision: $"
# $Source$

from pykg_config.version import Version

empty_raw_props = {'name': '',
                   'description': '',
                   'url': '',
                   'version': '',
                   'requires': '',
                   'requires.private': '',
                   'conflicts': '',
                   'cflags': '',
                   'libs': '',
                   'libs.private': ''}
empty_processed_props = {'name': '',
                         'description': 'No description',
                         'url': '',
                         'version': Version(),
                         'requires': [],
                         'requires.private': [],
                         'conflicts': [],
                         'include_dirs': [],
                         'other_cflags': [],
                         'libs': [],
                         'libpaths': [],
                         'otherlibs': [],
                         'private.libs': [],
                         'private.libpaths': [],
                         'private.otherlibs': []}


# vim: tw=79

